@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "parse.h"
 #include <stdio.h>
@@ -7,6 +10,21 @@
 #include <ctype.h>
 #include "popt.h"
 #include <sys/wait.h>
+#include <sys/types.h>
+
+/* since we never use getc_unlocked now we should really
+ * do something to speed up read_one_line
+ */
+#  define flockfile(f) (void)1
+#  define funlockfile(f) (void)1
+#  define getc_unlocked(f) getc(f)
+
+#ifdef NATIVE_WIN32
+
+#define STRICT
+#include <windows.h>
+
+#endif
 
 /**
  * Read an entire line from a file into a buffer. Lines may
@@ -164,14 +182,12 @@ trim_and_sub (Package *pkg, const char *str, const char *path)
 
           ++p; /* past brace */
           
-          varval = NULL;
-          if (pkg->vars != NULL)
-            varval = g_hash_table_lookup (pkg->vars, varname);
-
+          varval = package_get_var (pkg, varname);
+          
           if (varval == NULL)
             {
-              fprintf (stderr, "Variable '%s' not defined in '%s'\n",
-                       varname, path);
+              verbose_error ("Variable '%s' not defined in '%s'\n",
+                             varname, path);
               
               exit (1);
             }
@@ -200,7 +216,7 @@ parse_name (Package *pkg, const char *str, const char *path)
 {
   if (pkg->name)
     {
-      fprintf (stderr, "Name field occurs twice in '%s'\n", path);
+      verbose_error ("Name field occurs twice in '%s'\n", path);
 
       exit (1);
     }
@@ -213,7 +229,7 @@ parse_version (Package *pkg, const char *str, const char *path)
 {
   if (pkg->version)
     {
-      fprintf (stderr, "Version field occurs twice in '%s'\n", path);
+      verbose_error ("Version field occurs twice in '%s'\n", path);
 
       exit (1);
     }
@@ -226,7 +242,7 @@ parse_description (Package *pkg, const char *str, const char *path)
 {
   if (pkg->description)
     {
-      fprintf (stderr, "Description field occurs twice in '%s'\n", path);
+      verbose_error ("Description field occurs twice in '%s'\n", path);
 
       exit (1);
     }
@@ -258,7 +274,7 @@ typedef enum
 #define PARSE_SPEW 0
 
 static GSList*
-split_module_list (Package *pkg, const char *str, const char *path)
+split_module_list (const char *str, const char *path)
 {
   GSList *retval = NULL;
   const char *p;
@@ -372,19 +388,14 @@ split_module_list (Package *pkg, const char *str, const char *path)
   return retval;
 }
 
-static GSList*
+GSList*
 parse_module_list (Package *pkg, const char *str, const char *path)
 {
-  char *trimmed;
   GSList *split;
   GSList *iter;
   GSList *retval = NULL;
-  
-  trimmed = trim_and_sub (pkg, str, path);
 
-  split = split_module_list (pkg, trimmed, path);
-
-  g_free (trimmed);
+  split = split_module_list (str, path);
   
   iter = split;
   while (iter != NULL)
@@ -416,8 +427,7 @@ parse_module_list (Package *pkg, const char *str, const char *path)
 
       if (*start == '\0')
         {
-          fprintf (stderr,
-                   "Empty package name in Requires or Conflicts in file '%s'\n", path);
+          verbose_error ("Empty package name in Requires or Conflicts in file '%s'\n", path);
           
           exit (1);
         }
@@ -451,8 +461,7 @@ parse_module_list (Package *pkg, const char *str, const char *path)
             ver->comparison = NOT_EQUAL;
           else
             {
-              fprintf (stderr,
-                       "Unknown version comparison operator '%s' after package name '%s' in file '%s'\n", start, ver->name, path);
+              verbose_error ("Unknown version comparison operator '%s' after package name '%s' in file '%s'\n", start, ver->name, path);
               
               exit (1);
             }
@@ -471,8 +480,7 @@ parse_module_list (Package *pkg, const char *str, const char *path)
       
       if (ver->comparison != ALWAYS_MATCH && *start == '\0')
         {
-          fprintf (stderr,
-                   "Comparison operator but no version after package name '%s' in file '%s'\n", ver->name, path);
+          verbose_error ("Comparison operator but no version after package name '%s' in file '%s'\n", ver->name, path);
           
           exit (1);
         }
@@ -500,15 +508,18 @@ parse_requires (Package *pkg, const char *str, const char *path)
 {
   GSList *parsed;
   GSList *iter;
+  char *trimmed;
   
   if (pkg->requires)
     {
-      fprintf (stderr, "Requires field occurs twice in '%s'\n", path);
+      verbose_error ("Requires field occurs twice in '%s'\n", path);
 
       exit (1);
     }
 
-  parsed = parse_module_list (pkg, str, path);
+  trimmed = trim_and_sub (pkg, str, path);
+  parsed = parse_module_list (pkg, trimmed, path);
+  g_free (trimmed);
   
   iter = parsed;
   while (iter != NULL)
@@ -520,8 +531,8 @@ parse_requires (Package *pkg, const char *str, const char *path)
 
       if (req == NULL)
         {
-          fprintf (stderr, "Package '%s', required by '%s', not found\n",
-                   ver->name, pkg->name ? pkg->name : path);
+          verbose_error ("Package '%s', required by '%s', not found\n",
+                         ver->name, pkg->name ? pkg->name : path);
           
           exit (1);
         }
@@ -546,15 +557,18 @@ parse_conflicts (Package *pkg, const char *str, const char *path)
 {
   GSList *parsed;
   GSList *iter;
+  char *trimmed;
   
   if (pkg->conflicts)
     {
-      fprintf (stderr, "Conflicts field occurs twice in '%s'\n", path);
+      verbose_error ("Conflicts field occurs twice in '%s'\n", path);
 
       exit (1);
     }
 
-  pkg->conflicts = parse_module_list (pkg, str, path);
+  trimmed = trim_and_sub (pkg, str, path);
+  pkg->conflicts = parse_module_list (pkg, trimmed, path);
+  g_free (trimmed);
 }
 
 static void
@@ -571,7 +585,7 @@ parse_libs (Package *pkg, const char *str, const char *path)
   
   if (pkg->l_libs || pkg->L_libs || pkg->other_libs)
     {
-      fprintf (stderr, "Libs field occurs twice in '%s'\n", path);
+      verbose_error ("Libs field occurs twice in '%s'\n", path);
 
       exit (1);
     }
@@ -582,7 +596,7 @@ parse_libs (Package *pkg, const char *str, const char *path)
 
   if (result < 0)
     {
-      fprintf (stderr, "Couldn't parse Libs field into an argument vector: %s\n",
+      verbose_error ("Couldn't parse Libs field into an argument vector: %s\n",
                poptStrerror (result));
 
       exit (1);
@@ -677,7 +691,7 @@ parse_cflags (Package *pkg, const char *str, const char *path)
   
   if (pkg->I_cflags || pkg->other_cflags)
     {
-      fprintf (stderr, "Cflags field occurs twice in '%s'\n", path);
+      verbose_error ("Cflags field occurs twice in '%s'\n", path);
 
       exit (1);
     }
@@ -688,8 +702,8 @@ parse_cflags (Package *pkg, const char *str, const char *path)
 
   if (result < 0)
     {
-      fprintf (stderr, "Couldn't parse Cflags field into an argument vector: %s\n",
-               poptStrerror (result));
+      verbose_error ("Couldn't parse Cflags field into an argument vector: %s\n",
+                     poptStrerror (result));
 
       exit (1);
     }
@@ -752,9 +766,11 @@ parse_line (Package *pkg, const char *untrimmed, const char *path)
   char *str;
   char *p;
   char *tag;
+
+  debug_spew ("  line>%s\n", untrimmed);
   
   str = trim_string (untrimmed);
-
+  
   if (*str == '\0')
     return; /* empty line */
   
@@ -795,8 +811,8 @@ parse_line (Package *pkg, const char *untrimmed, const char *path)
         parse_conflicts (pkg, p, path);
       else
         {
-          fprintf (stderr, "Unknown keyword '%s' in '%s'\n",
-                   tag, path);
+          verbose_error ("Unknown keyword '%s' in '%s'\n",
+                         tag, path);
 
           exit (1);
         }
@@ -804,6 +820,9 @@ parse_line (Package *pkg, const char *untrimmed, const char *path)
   else if (*p == '=')
     {
       /* variable */
+      char *varname;
+      char *varval;
+      
       ++p;
       while (*p && isspace (*p))
         ++p;
@@ -813,14 +832,19 @@ parse_line (Package *pkg, const char *untrimmed, const char *path)
 
       if (g_hash_table_lookup (pkg->vars, tag))
         {
-          fprintf (stderr, "Duplicate definition of variable '%s' in '%s'\n",
-                   tag, path);
+          verbose_error ("Duplicate definition of variable '%s' in '%s'\n",
+                         tag, path);
 
           exit (1);
         }
 
-      g_hash_table_insert (pkg->vars, g_strdup (tag),
-                           trim_and_sub (pkg, p, path));
+      varname = g_strdup (tag);
+      varval = trim_and_sub (pkg, p, path);     
+
+      debug_spew (" Variable declaration, '%s' has value '%s'\n",
+                  varname, varval);
+      g_hash_table_insert (pkg->vars, varname, varval);
+  
     }
   
   g_free (str);
@@ -833,26 +857,46 @@ parse_package_file (const char *path)
   FILE *f;
   Package *pkg;
   GString *str;
+  gboolean one_line = FALSE;
   
   f = fopen (path, "r");
 
   if (f == NULL)
     {
-      fprintf (stderr, "Failed to open '%s': %s\n",
-               path, strerror (errno));
+      verbose_error ("Failed to open '%s': %s\n",
+                     path, strerror (errno));
       
       return NULL;
     }
+
+  debug_spew ("Parsing package file '%s'\n", path);
   
   pkg = g_new0 (Package, 1);
+
+  if (path)
+    {
+      pkg->pcfiledir = g_dirname (path);
+    }
+  else
+    {
+      debug_spew ("No pcfiledir determined for package\n");
+      pkg->pcfiledir = g_strdup ("???????");
+    }
+  
   str = g_string_new ("");
 
   while (read_one_line (f, str))
     {
+      one_line = TRUE;
+      
       parse_line (pkg, str->str, path);
 
       g_string_truncate (str, 0);
     }
+
+  if (!one_line)
+    verbose_error ("Package file '%s' appears to be empty\n",
+                   path);
   
   return pkg;
 }
@@ -905,12 +949,16 @@ get_compat_package (const char *name)
 {
   Package *pkg;
 
+  debug_spew ("Looking for '%s' using old-style -config scripts\n", name);
+  
   pkg = g_new0 (Package, 1);
   
   if (strcmp (name, "glib") == 0)
     {
       char *output;
 
+      debug_spew ("Calling glib-config\n");
+      
       pkg->version = backticks ("glib-config --version");
       if (pkg->version == NULL)
         {
@@ -936,6 +984,8 @@ get_compat_package (const char *name)
     {
       char *output;
 
+      debug_spew ("Calling gtk-config\n");
+      
       pkg->version = backticks ("gtk-config --version");
       if (pkg->version == NULL)
         {
@@ -961,6 +1011,8 @@ get_compat_package (const char *name)
     {
       char *output;
 
+      debug_spew ("Calling imlib-config\n");
+      
       pkg->version = backticks ("imlib-config --version");
       if (pkg->version == NULL)
         {
@@ -986,6 +1038,8 @@ get_compat_package (const char *name)
     {
       char *output;
       char *p;
+
+      debug_spew ("Calling orbit-config\n");
       
       output = backticks ("orbit-config --version");
       
@@ -1034,6 +1088,8 @@ get_compat_package (const char *name)
     {
       char *output;
       char *p;
+
+      debug_spew ("Calling orbit-config\n");
       
       output = backticks ("orbit-config --version");
       
@@ -1085,6 +1141,8 @@ get_compat_package (const char *name)
       char *p;
       char *command;
 
+      debug_spew ("Calling gnome-config\n");
+      
       /* Annoyingly, --modversion doesn't return a failure
        * code if the lib is unknown, so we have to use --libs
        * for that.
