@@ -1,6 +1,18 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "pkg.h"
 #include "parse.h"
 
+#ifdef HAVE_ALLOCA_H
+# include <alloca.h>
+#else
+# ifdef _AIX
+#  pragma alloca
+# endif
+#endif
+
 #include <dirent.h>
 #include <string.h>
 #include <errno.h>
@@ -12,8 +24,11 @@ static void verify_package (Package *pkg);
 
 static GHashTable *packages = NULL;
 static GHashTable *locations = NULL;
+static GHashTable *globals = NULL;
 static GSList *search_dirs = NULL;
 
+gboolean disable_uninstalled = FALSE;
+
 void
 add_search_dir (const char *path)
 {
@@ -36,6 +51,22 @@ ends_in_dotpc (const char *str)
     return FALSE;
 }
 
+/* strlen ("uninstalled") */
+#define UNINSTALLED_LEN 11 
+
+static gboolean
+ends_in_uninstalled (const char *str)
+{
+  int len = strlen (str);
+  
+  if (len > UNINSTALLED_LEN &&
+      strcmp ((str + len - UNINSTALLED_LEN), "uninstalled") == 0)
+    return TRUE;
+  else
+    return FALSE;
+}
+
+
 /* Look for .pc files in the given directory and add them into
  * locations, ignoring duplicates
  */
@@ -51,11 +82,13 @@ scan_dir (const char *dirname)
 
   if (!dir)
     {
-      fprintf (stderr, "Warning: cannot open directory '%s' in package search path: %s\n",
-               dirname, g_strerror (errno));
+      debug_spew ("Cannot open directory '%s' in package search path: %s\n",
+                  dirname, g_strerror (errno));
       return;
     }
 
+  debug_spew ("Scanning directory '%s'\n", dirname);
+  
   while ((dent = readdir (dir)))
     {
       int len = strlen (dent->d_name);
@@ -63,11 +96,15 @@ scan_dir (const char *dirname)
       if (ends_in_dotpc (dent->d_name))
         {
           char *pkgname = malloc (len - 2);
+
+          debug_spew ("File '%s' appears to be a .pc file\n", dent->d_name);
+          
 	  strncpy (pkgname, dent->d_name, len - EXT_LEN);
           pkgname[len-EXT_LEN] = '\0';
 
           if (g_hash_table_lookup (locations, pkgname))
             {
+              debug_spew ("File '%s' ignored, we already know about package '%s'\n", dent->d_name, pkgname);
               g_free (pkgname);
             }
           else
@@ -78,8 +115,16 @@ scan_dir (const char *dirname)
               strcpy (filename + dirnamelen + 1, dent->d_name);
               
               g_hash_table_insert (locations, pkgname, filename);
+
+              debug_spew ("Will find package '%s' in file '%s'\n",
+                          pkgname, filename);
             }
         }
+      else
+        {
+          debug_spew ("Ignoring file '%s' in search directory; not a .pc file\n",
+                      dent->d_name);
+        }
     }
 }
 
@@ -115,22 +160,49 @@ file_readable (const char *path)
 }
 
 
-Package *
-get_package (const char *name)
+static Package *
+internal_get_package (const char *name, gboolean warn)
 {
   Package *pkg = NULL;
   const char *location;
-
+  
   pkg = g_hash_table_lookup (packages, name);
 
   if (pkg)
     return pkg;
+
+  debug_spew ("Looking for package '%s'\n", name);
   
   /* treat "name" as a filename if it ends in .pc and exists */
   if ( ends_in_dotpc (name) )
-    location = name;
+    {
+      debug_spew ("Considering '%s' to be a filename rather than a package name\n", name);
+      location = name;
+    }
   else
-    location = g_hash_table_lookup (locations, name);
+    {
+      /* See if we should auto-prefer the uninstalled version */
+      if (!disable_uninstalled &&
+          !ends_in_uninstalled (name))
+        {
+          char *un;
+
+          un = g_strconcat (name, "-uninstalled", NULL);
+
+          pkg = internal_get_package (un, FALSE);
+
+          g_free (un);
+          
+          if (pkg)
+            {
+              debug_spew ("Preferring uninstalled version of package '%s'\n", name);
+              return pkg;
+            }
+        }
+      
+      location = g_hash_table_lookup (locations, name);
+      debug_spew ("Reading '%s' from file '%s'\n", name, location);
+    }
   
   if (location == NULL)
     {
@@ -138,21 +210,30 @@ get_package (const char *name)
       
       if (pkg == NULL)
         {
-          fprintf (stderr,
-                   "Package %s was not found in the pkg-config search path.\n"
-                   "Perhaps you should add the directory containing `%s.pc'\n"
-                   "to the PKG_CONFIG_PATH environment variable\n",
-                   name, name);
+          if (warn)
+            verbose_error ("Package %s was not found in the pkg-config search path.\n"
+                           "Perhaps you should add the directory containing `%s.pc'\n"
+                           "to the PKG_CONFIG_PATH environment variable\n",
+                           name, name);
         }
+
+      debug_spew ("Returning values for '%s' from a legacy -config script\n",
+                  name);
       
       return pkg;
     }
 
   pkg = parse_package_file (location);
-
+  
   if (pkg == NULL)
-    return NULL;
+    {
+      debug_spew ("Failed to parse '%s'\n", location);
+      return NULL;
+    }
 
+  if (strstr (location, "uninstalled.pc"))
+    pkg->uninstalled = TRUE;
+  
   if (location != name)
     pkg->key = g_strdup (name);
   else
@@ -171,12 +252,21 @@ get_package (const char *name)
     }
 
   verify_package (pkg);
+
+  debug_spew ("Adding '%s' to list of known packages, returning as package '%s'\n",
+              pkg->key, name);
   
   g_hash_table_insert (packages, pkg->key, pkg);
 
   return pkg;
 }
 
+Package *
+get_package (const char *name)
+{
+  return internal_get_package (name, TRUE);
+}
+
 static GSList*
 string_list_strip_duplicates (GSList *list)
 {
@@ -337,6 +427,36 @@ verify_package (Package *pkg)
   GSList *iter;
   GSList *requires_iter;
   GSList *conflicts_iter;  
+
+  /* Be sure we have the required fields */
+
+  if (pkg->key == NULL)
+    {
+      fprintf (stderr,
+               "Internal pkg-config error, package with no key, please file a bug report\n");
+      exit (1);
+    }
+  
+  if (pkg->name == NULL)
+    {
+      verbose_error ("Package '%s' has no Name: field\n",
+                     pkg->key);
+      exit (1);
+    }
+
+  if (pkg->version == NULL)
+    {
+      verbose_error ("Package '%s' has no Version: field\n",
+                     pkg->name);
+      exit (1);
+    }
+
+  if (pkg->description == NULL)
+    {
+      verbose_error ("Package '%s' has no Description: field\n",
+                     pkg->description);
+      exit (1);
+    }
   
   /* Make sure we have the right version for all requirements */
 
@@ -355,13 +475,12 @@ verify_package (Package *pkg)
         {
           if (!version_test (ver->comparison, req->version, ver->version))
             {
-              fprintf (stderr,
-                       "Package '%s' requires '%s %s %s' but version of %s is %s\n",
-                       pkg->name, req->key,
-                       comparison_to_str (ver->comparison),
-                       ver->version,
-                       req->name,
-                       req->version);
+              verbose_error ("Package '%s' requires '%s %s %s' but version of %s is %s\n",
+                             pkg->name, req->key,
+                             comparison_to_str (ver->comparison),
+                             ver->version,
+                             req->name,
+                             req->version);
 
               exit (1);
             }
@@ -392,15 +511,14 @@ verify_package (Package *pkg)
                             req->version,
                             ver->version))
             {
-              fprintf (stderr,
-                       "Version %s of %s creates a conflict.\n"
-                       "(%s %s %s conflicts with %s %s)\n",
-                       req->version, req->name,
-                       ver->name,
-                       comparison_to_str (ver->comparison),
-                       ver->version,
-                       ver->owner->name,
-                       ver->owner->version);
+              verbose_error ("Version %s of %s creates a conflict.\n"
+                             "(%s %s %s conflicts with %s %s)\n",
+                             req->version, req->name,
+                             ver->name,
+                             comparison_to_str (ver->comparison),
+                             ver->version,
+                             ver->owner->name,
+                             ver->owner->version);
 
               exit (1);
             }
@@ -697,14 +815,43 @@ packages_get_all_cflags (GSList     *pkgs)
   return retval;
 }
 
+
+void
+define_global_variable (const char *varname,
+                        const char *varval)
+{
+  if (globals == NULL)
+    globals = g_hash_table_new (g_str_hash, g_str_equal);
+
+  if (g_hash_table_lookup (globals, varname))
+    {
+      verbose_error ("Variable '%s' defined twice globally\n", varname);
+      exit (1);
+    }
+  
+  g_hash_table_insert (globals, g_strdup (varname), g_strdup (varval));
+      
+  debug_spew ("Global variable definition '%s' = '%s'\n",
+              varname, varval);
+}
+
 char *
 package_get_var (Package *pkg,
                  const char *var)
 {
-  if (pkg->vars == NULL)
-    return NULL;
-  else 
-    return g_strdup (g_hash_table_lookup (pkg->vars, var));
+  char *varval = NULL;
+
+  if (globals)
+    varval = g_hash_table_lookup (globals, var);
+  
+  if (varval == NULL && pkg->vars)
+    varval = g_strdup (g_hash_table_lookup (pkg->vars, var));
+
+  /* Magic "pcfiledir" variable */
+  if (varval == NULL && pkg->pcfiledir && strcmp (var, "pcfiledir") == 0)
+    varval = g_strdup (pkg->pcfiledir);
+
+  return varval;
 }
 
 char *
